﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics;
using Infragistics.Controls;

namespace XamRichTextEditorStudio
{

	public enum RichTextEditorStudioCommandType
	{
		CloseDocument,
		NewDocument,
		OpenDocument,
		SaveDocument
	}

	#region RichTextEditorStudioCommandBase Class
	public class RichTextEditorStudioCommand : CommandBase
	{
		#region Member Variables

		private RichTextEditorStudioCommandType _commandType;

		#endregion Member Variables

		#region Constructor
		public RichTextEditorStudioCommand(RichTextEditorStudioCommandType commandType)
		{
			this._commandType = commandType;
		}
		#endregion //Constructor

		#region Overrides

		#region CanExecute
		public override bool CanExecute(object parameter)
		{
			RichTextEditorStudioUserControl studioUserControl = parameter as RichTextEditorStudioUserControl;
			if (studioUserControl != null)
			{

				switch (this._commandType)
				{
					case RichTextEditorStudioCommandType.CloseDocument:
						return true;
					case RichTextEditorStudioCommandType.NewDocument:
						return true;
					case RichTextEditorStudioCommandType.OpenDocument:
						return true;
					case RichTextEditorStudioCommandType.SaveDocument:
						return true;
				}

				if (null != this.CommandSource)
					this.CommandSource.Handled = true;
			}

			return false;
		}
		#endregion //CanExecute

		#region Execute
		public override void Execute(object parameter)
		{
			RichTextEditorStudioUserControl studioUserControl = parameter as RichTextEditorStudioUserControl;
			if (studioUserControl != null)
			{
				switch (this._commandType)
				{
					case RichTextEditorStudioCommandType.CloseDocument:
						studioUserControl.CloseDocument();
						break;
					case RichTextEditorStudioCommandType.NewDocument:
						studioUserControl.NewDocument();
						break;
					case RichTextEditorStudioCommandType.OpenDocument:
						studioUserControl.OpenDocument();
						break;
					case RichTextEditorStudioCommandType.SaveDocument:
						studioUserControl.SaveDocument();
						break;
				}

				if (null != this.CommandSource)
					this.CommandSource.Handled = true;
			}

			base.Execute(parameter);
		}
		#endregion // Execute

		#endregion // Overrides
	}
	#endregion // RichTextEditorStudioCommandBase Class

	#region RichTextEditorStudioCommandSource Class
	public class RichTextEditorStudioCommandSource : CommandSource
	{
		public RichTextEditorStudioCommandType CommandType { get; set; }

		protected override ICommand ResolveCommand()
		{
			return new RichTextEditorStudioCommand(this.CommandType);
		}
	}

	#endregion //RichTextEditorStudioCommandSource Class
}
