using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System;

namespace XamRichTextEditorStudio
{
	public class FontInfo : INotifyPropertyChanged
	{
		#region Member Variables

		private ObservableCollection<FontNameInfo> _fontNames;
		private ObservableCollection<FontSizeInfo> _fontSizes;

		#endregion //Member Variables

		#region Properties

		#region FontNames
		public ObservableCollection<FontNameInfo> FontNames
		{
			get
			{
				if (this._fontNames == null)
				{
					this._fontNames =
						new ObservableCollection<FontNameInfo>
						{
							new FontNameInfo  { Value = "Arial" },
							new FontNameInfo  { Value = "Calibri" },
							new FontNameInfo  { Value = "Comic Sans MS" },
							new FontNameInfo  { Value = "Consolas" },
							new FontNameInfo  { Value = "Courier New" },
							new FontNameInfo  { Value = "Lucida Calligraphy" },
							new FontNameInfo  { Value = "Lucida Handwriting" },
							new FontNameInfo  { Value = "Segoe UI" },
							new FontNameInfo  { Value = "Tahoma" },
							new FontNameInfo  { Value = "Times New Roman" },
							new FontNameInfo  { Value = "Trebuchet MS" },
							new FontNameInfo  { Value = "Verdana" }
						};
				}

				return this._fontNames;
			}
		}
		#endregion //FontNames

		#region FontSizes
		public ObservableCollection<FontSizeInfo> FontSizes
		{
			get
			{
				if (this._fontSizes == null)
				{
					this._fontSizes =
						new ObservableCollection<FontSizeInfo>
						{
							new FontSizeInfo  { Value = 8.0, Description = "8", },
							new FontSizeInfo  { Value = 9.0, Description = "9", },
							new FontSizeInfo  { Value = 10.0, Description = "10", },
							new FontSizeInfo  { Value = 11.0, Description = "11", },
							new FontSizeInfo  { Value = 12.0, Description = "12", },
							new FontSizeInfo  { Value = 14.0, Description = "14", },
							new FontSizeInfo  { Value = 16.0, Description = "16", },
							new FontSizeInfo  { Value = 18.0, Description = "18", },
							new FontSizeInfo  { Value = 20.0, Description = "20", },
							new FontSizeInfo  { Value = 22.0, Description = "22", },
							new FontSizeInfo  { Value = 24.0, Description = "24", },
							new FontSizeInfo  { Value = 26.0, Description = "26", },
							new FontSizeInfo  { Value = 28.0, Description = "28", },
							new FontSizeInfo  { Value = 36.0, Description = "36", },
							new FontSizeInfo  { Value = 48.0, Description = "48", },
							new FontSizeInfo  { Value = 72.0, Description = "72", },
						};
				}

				return this._fontSizes;
			}
		}
		#endregion //FontSizes

		#endregion //Properties

		#region Methods

		#region IndexOfFontName
		public int IndexOfFontName(string fontName)
		{
			for (int i = 0; i < this.FontNames.Count; i++)
			{
				if (this.FontNames[i].Description == fontName)
					return i;
			}

			return -1;
		}
		#endregion //IndexOfFontName

		#region IndexOfFontSize
		public int IndexOfFontSize(double fontSize)
		{
			for (int i = 0; i < this.FontSizes.Count; i++)
			{
				if (this.FontSizes[i].Value == fontSize)
					return i;
			}

			return -1;
		}
		#endregion //IndexOfFontSize

		#endregion //Methods

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;
		private void RaisePropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		#endregion
	}

	#region FontNameInfo Class
	public class FontNameInfo : INotifyPropertyChanged
	{
		#region Member Variables
		private string _value;
		#endregion //Member Variables

		#region Base Class Overrides

		#region ToString
		public override string ToString()
		{
			return this.Description;
		}
		#endregion //ToString

		#endregion //Base Class Overrides

		#region Properties

		#region Description
		public string Description
		{
			get { return this._value; }
		}
		#endregion //Value

		#region Value
		public string Value
		{
			get { return this._value; }
			set
			{
				if (this._value != value)
				{
					this._value = value;
					OnPropertyChanged("Value");
					OnPropertyChanged("Description");
				}
			}
		}
		#endregion //Value

		#endregion //Properties

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
		#endregion //INotifyPropertyChanged Members
	}
	#endregion //FontNameInfo Class

	#region FontSizeInfo Class
	public class FontSizeInfo : INotifyPropertyChanged
	{
		#region Member Variables
		private double _value;
		private string _description;
		#endregion //Member Variables

		#region Base Class Overrides

		#region ToString
		public override string ToString()
		{
			return this.Description;
		}
		#endregion //ToString

		#endregion //Base Class Overrides

		#region Properties

		#region Value
		public double Value
		{
			get { return this._value; }
			set
			{
				if (this._value != value)
				{
					this._value = value;
					OnPropertyChanged("Value");
				}
			}
		}
		#endregion //Value

		#region Description
		public string Description
		{
			get { return this._description; }
			set
			{
				if (this._description != value)
				{
					this._description = value;
					OnPropertyChanged("Description");
				}
			}
		}
		#endregion //Description

		#endregion //Properties

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
		#endregion //INotifyPropertyChanged Members
	}
	#endregion //FontSizeInfo Class
}
