﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics;
using Infragistics.Controls;
using Infragistics.Controls.Editors;
using Infragistics.Controls.Editors.Primitives;
using Infragistics.Documents.RichText;
using Infragistics.Windows.Editors;
using Infragistics.Windows.Ribbon;
using Infragistics.Windows.Ribbon.Events;

namespace XamRichTextEditorStudio
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : XamRibbonWindow, ICommandTarget
	{
		#region Member Variables

		private bool					_internalToolSettingsUpdateInProgress;
		private UnderlineType?			_lastSelectedUnderlineType = UnderlineType.Single;
		private Color					_lastSelectedTextForecolor = Colors.Black;
		private HighlightColor			_lastSelectedTextHighlightColor = HighlightColor.Yellow;
		private Color					_lastSelectedParagraphShadingColor = Colors.White;
		private string					_lastSelectedBulletStyleId;
		private string					_lastSelectedNumberingStyleId;

		#endregion //Member Variables

		#region Constructor
		public MainWindow()
		{
			InitializeComponent();

			this.Initialize();
		}
		#endregion //Constructor

		#region Properties

		#region RichTextEditor
		/// <summary>
		/// Returns the XamRichTextEditor control hosted in this window.
		/// </summary>
		public XamRichTextEditor RichTextEditor
		{
			get { return this.richTextEditorStudioUserControl.RichTextEditor; }
		}
		#endregion //RichTextEditor

		#region RichTextEditorStudioUserControl
		/// <summary>
		/// Returns the RichTextEditorStudioUserControl hosted in this window.
		/// </summary>
		public RichTextEditorStudioUserControl RichTextEditorStudioUserControl
		{
			get { return this.richTextEditorStudioUserControl; }
		}
		#endregion //RichTextEditorStudioUserControl

		#endregion //Properties

		#region Methods

		#region ClearBulletNumberingMenuToolItems
		private void ClearBulletNumberingMenuToolItems(MenuTool bulletOrNumberingTool, bool isBulletTool)
		{
			bulletOrNumberingTool.IsChecked = false;
			foreach (ToggleButtonTool toggleButtonTool in bulletOrNumberingTool.Items)
			{
				if (isBulletTool)
					toggleButtonTool.Click -= bullet_Click;
				else
					toggleButtonTool.Click -= numbering_Click;
			}

			bulletOrNumberingTool.Items.Clear();
		}
		#endregion //ClearBulletNumberingMenuToolItems

		#region Initialize
		private void Initialize()
		{
			// Set the DataContext to the window so elements in XAML can bind to the window's properties
			this.DataContext = this;

			// Register the XamRichTextEditor as a command target so that the RichTextEditorCommands can be processed.
			CommandSourceManager.RegisterCommandTarget(this.RichTextEditor);

			// Listen to the DocumentContentOrSelectionChanged event on the RichTextEditorStudioUserControl
			this.richTextEditorStudioUserControl.DocumentContentOrSelectionChanged += richTextEditorStudioUserControl_DocumentContentOrSelectionChanged;

			// Listen to the RichTextEditor ActiveDocumentViewChanged event
			this.RichTextEditor.ActiveDocumentViewChanged += RichTextEditor_ActiveDocumentViewChanged;

			// Listen to the RichTextEditor DocumentChanged event
			this.RichTextEditor.DocumentChanged += RichTextEditor_DocumentChanged;

			// Initialize Bullet/Numbering Tools.
			this.InitializeBulletAndNumberingTools();
		}
		#endregion //Initialize

		#region InitializeBulletAndNumberingTools
		private void InitializeBulletAndNumberingTools()
		{
			// Clear the existing tools (if any) in the bullet and numbering menus.
			this.ClearBulletNumberingMenuToolItems(this.bulletTool, true);
			this.ClearBulletNumberingMenuToolItems(this.numberingTool, false);

			// Add tools to the bullet and numbering menus for each list style in the document.
			ToggleButtonTool toggleButtonTool;

			foreach (ListStyle listStyle in this.RichTextEditor.Document.GetAvailableListStyles())
			{
				toggleButtonTool			= new ToggleButtonTool();
				toggleButtonTool.Caption	= listStyle.Name;
				toggleButtonTool.Tag		= listStyle.Id;

				if (listStyle.IsFirstLevelBullet)
				{
					toggleButtonTool.Click += bullet_Click;
					this.bulletTool.Items.Add(toggleButtonTool);

					// Consider the first bullet style we encounter to be the default bullet style in the UI.
					if (string.IsNullOrEmpty(this._lastSelectedBulletStyleId))
					{
						this._lastSelectedBulletStyleId = listStyle.Id;
						toggleButtonTool.IsChecked		= true;
					}
				}
				else
				{
					toggleButtonTool.Click += numbering_Click;
					this.numberingTool.Items.Add(toggleButtonTool);

					// Consider the first numbering style we encounter to be the the default numbering style in the UI.
					if (string.IsNullOrEmpty(this._lastSelectedNumberingStyleId))
					{
						this._lastSelectedNumberingStyleId	= listStyle.Id;
						toggleButtonTool.IsChecked			= true;
					}
				}
			}

			// Add a 'None' entry to the list of bullets.
			toggleButtonTool			= new ToggleButtonTool();
			toggleButtonTool.Caption	= "None";
			toggleButtonTool.Click		+= bullet_Click;
			this.bulletTool.Items.Add(toggleButtonTool);

			// Add a 'None' entry to the list of numbering.
			toggleButtonTool			= new ToggleButtonTool();
			toggleButtonTool.Caption	= "None";
			toggleButtonTool.Click		+= numbering_Click;
			this.numberingTool.Items.Add(toggleButtonTool);
		}
		#endregion //InitializeBulletAndNumberingTools

		#region UpdateBulletNumberingToolState
		private void UpdateBulletNumberingToolState(MenuTool bulletOrNumberingTool, string currentListStyleId)
		{
			bulletOrNumberingTool.IsChecked = false;
			foreach (ToggleButtonTool toggleButtonTool in bulletOrNumberingTool.Items)
			{
				toggleButtonTool.IsChecked = (string)toggleButtonTool.Tag == currentListStyleId;
				if (true	== toggleButtonTool.IsChecked && 
					false	== string.IsNullOrEmpty(currentListStyleId))
					bulletOrNumberingTool.IsChecked = true;
			}
		}
		#endregion //UpdateBulletNumberingToolState

		#region UpdateToolSettings
		private void UpdateToolSettings()
		{
			this._internalToolSettingsUpdateInProgress = true;
			
			try
			{
				CharacterSettings	characterSettings	= this.richTextEditorStudioUserControl.CurrentCharacterSettings;
				FontInfo			fontInfo			= this.xamRibbon1.Resources["fontInfo"] as FontInfo;
				if (null != fontInfo)
				{
					// FontName Tool
					if (null != characterSettings.FontSettings)
						this.fontNameTool.SelectedIndex = fontInfo.IndexOfFontName(characterSettings.FontSettings.Ascii.Value.Name);

					// FontSize Tool
					if (false == characterSettings.FontSize.HasValue)
						this.fontSizeTool.SelectedIndex = -1;
					else
						this.fontSizeTool.SelectedIndex = fontInfo.IndexOfFontSize(Math.Floor(characterSettings.FontSize.Value.Points));

					// FontUnderline Tool - Set the checked state of the FontUnderline Tool if the current character settings matches
					// the last selected underline type..
					if (false == characterSettings.UnderlineType.HasValue || false == this._lastSelectedUnderlineType.HasValue)
						this.fontUnderlineTool.IsChecked = false;
					else
						this.fontUnderlineTool.IsChecked = characterSettings.UnderlineType.Value == this._lastSelectedUnderlineType;

					// ForecolorText Tool - Set the checked state of the ForecolorTextTool if the current character settings matches
					// the last selected foreColorText.
					if (null == characterSettings.Color || false == characterSettings.Color.Color.HasValue)
						this.forecolorTextTool.IsChecked = false;
					else
						this.forecolorTextTool.IsChecked = characterSettings.Color.Color.Value == this._lastSelectedTextForecolor;

					// HighlightColor Tool - Set the checked state of the HighlightColor if the current character settings matches
					// the last selected HighlightColor.
					if (false == characterSettings.HighlightColor.HasValue)
						this.highlightColorTool.IsChecked = false;
					else
						this.highlightColorTool.IsChecked = characterSettings.HighlightColor == this._lastSelectedTextHighlightColor;

					// ParagraphShading Tool - Set the checked state of the ParagraphShading Tool if the current character settings matches
					// the last selected ParagraphShading color.
					ParagraphSettings paragraphSettings = this.RichTextEditorStudioUserControl.CurrentParagraphSettings;
					if (false == paragraphSettings.Shading.HasValue || false == paragraphSettings.Shading.Value.Background.Color.HasValue)
						this.paragraphShadingTool.IsChecked = false;
					else
						this.paragraphShadingTool.IsChecked = paragraphSettings.Shading.Value.Background.Color == this._lastSelectedParagraphShadingColor;

					// Bullet/Numbering Tools - Set the checked state of the Bullet/Numbering Tools if the current selection
					// has a list style Id applied.  Also update the checked state of the tools in the 
					// dropdown to reflect the current list style.
					string listStyleId = this.RichTextEditor.Document.GetCommonParagraphListStyle(this.RichTextEditor.Selection.DocumentSpan);
					this.UpdateBulletNumberingToolState(this.bulletTool, listStyleId);
					this.UpdateBulletNumberingToolState(this.numberingTool, listStyleId);
				}
			}
			finally
			{
				this._internalToolSettingsUpdateInProgress = false;
			}
		}
		#endregion //UpdateToolSettings

		#endregion //Methods

		#region ICommandTarget
		public object GetParameter(CommandSource source)
		{
			return this.richTextEditorStudioUserControl;
		}

		public bool SupportsCommand(ICommand command)
		{
			return command is RichTextEditorStudioCommand;
		}
		#endregion //ICommandTarget

		#region Event Handlers

		#region bullet_Click
		private void bullet_Click(object sender, RoutedEventArgs e)
		{
			ToggleButtonTool clickedTool = sender as ToggleButtonTool;

			if (null != clickedTool)
			{
				string styleId = (string)clickedTool.Tag;
				if (string.IsNullOrEmpty(styleId))
					this.RichTextEditor.Selection.ClearParagraphListStyle();
				else
					this.RichTextEditor.Selection.ApplyParagraphListStyle(styleId);

				this._lastSelectedBulletStyleId = styleId;
			}
		}
		#endregion //bullet_Click

		#region bulletTool_Click
		private void bulletTool_Click(object sender, RoutedEventArgs e)
		{
			if (this.bulletTool.IsChecked)
				this.RichTextEditor.Selection.ApplyParagraphListStyle(this._lastSelectedBulletStyleId);
			else
				this.RichTextEditor.Selection.ClearParagraphListStyle();
		}
		#endregion //bulletTool_Click

		#region fontNameTool_ValueChanged
		private void fontNameTool_ValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
		{
			if (true == this._internalToolSettingsUpdateInProgress)
				return;

			ComboEditorTool comboEditorTool = sender as ComboEditorTool;
			FontNameInfo fontNameInfo = comboEditorTool.SelectedItem as FontNameInfo;

			if (null != fontNameInfo)
				this.RichTextEditor.Selection.ApplyFont(fontNameInfo.Value);
		}
		#endregion //fontNameTool_ValueChanged

		#region fontSizeTool_ValueChanged
		private void fontSizeTool_ValueChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
		{
			if (true == this._internalToolSettingsUpdateInProgress)
				return;

			ComboEditorTool comboEditorTool = sender as ComboEditorTool;
			FontSizeInfo fontSizeInfo = comboEditorTool.SelectedItem as FontSizeInfo;

			if (null != fontSizeInfo)
				this.RichTextEditor.Selection.ApplyFontSize(fontSizeInfo.Value);
		}
		#endregion //fontSizeTool_ValueChanged

		#region fontUnderlineTool_Opening
		private void fontUnderlineTool_Opening(object sender, ToolOpeningEventArgs e)
		{
			// Initialize the checked state of the togglebutton tools in the menu based on the current
			// CharacterSettings Underline
			CharacterSettings characterSettings = this.richTextEditorStudioUserControl.CurrentCharacterSettings;
			UnderlineType? underlineType = characterSettings.UnderlineType;

			foreach (ToggleButtonTool tool in this.fontUnderlineTool.Items)
			{
				tool.IsChecked = underlineType.HasValue &&
								 underlineType == RichTextEditorStudioUserControl.UnderlineTypeFromDescription(tool.Caption);
			}
		}
		#endregion //fontUnderlineTool_Opening

		#region fontUnderlineTool_Click
		private void fontUnderlineTool_Click(object sender, RoutedEventArgs e)
		{
			if (false == this.fontUnderlineTool.IsChecked)
				this.RichTextEditor.Selection.ApplyFontUnderlineFormatting(UnderlineType.None);
			else
			if (this._lastSelectedUnderlineType.HasValue)
				this.RichTextEditor.Selection.ApplyFontUnderlineFormatting(this._lastSelectedUnderlineType.Value);
		}
		#endregion //fontUnderlineTool_Click

		#region fontUnderline_Click
		private void fontUnderline_Click(object sender, RoutedEventArgs e)
		{
			ToggleButtonTool clickedTool			= sender as ToggleButtonTool;
			UnderlineType	 selectedUnderlineType	= RichTextEditorStudioUserControl.UnderlineTypeFromDescription(clickedTool.Caption);

			this._lastSelectedUnderlineType = selectedUnderlineType;

			this.RichTextEditor.Selection.ApplyFontUnderlineFormatting(selectedUnderlineType);
		}
		#endregion //fontUnderline_Click

		#region forecolorTextTool_Opening
		private void forecolorTextTool_Opening(object sender, ToolOpeningEventArgs e)
		{
			// Initialize the checked state of the GalleryItems based on the current
			// CharacterSettings Color
			CharacterSettings	characterSettings	= this.richTextEditorStudioUserControl.CurrentCharacterSettings;
			Color?				colorSetting		= null == characterSettings.Color || false == characterSettings.Color.Color.HasValue ? 
				null : characterSettings.Color.Color;

			GalleryTool			galleryTool			= this.forecolorTextTool.Items[0] as GalleryTool;
			foreach (GalleryItem galleryItem in galleryTool.Items)
			{
				galleryItem.IsSelected = null != colorSetting &&
								 colorSetting == (Color)galleryItem.Tag;
			}
		}
		#endregion //forecolorTextTool_Opening

		#region forecolorTextTool_Click
		private void forecolorTextTool_Click(object sender, RoutedEventArgs e)
		{
			if (false == this.forecolorTextTool.IsChecked)
				this.RichTextEditor.Selection.ApplyTextForecolor(ColorInfo.Automatic.Color.Value);
			else
				this.RichTextEditor.Selection.ApplyTextForecolor(this._lastSelectedTextForecolor);
		}
		#endregion //forecolorTextTool_Click

		#region forecolorText_ColorClick
		private void forecolorText_ColorClick(object sender, GalleryItemEventArgs e)
		{
			if (e.Item == null || e.Item.Tag == null)
				return;

			Color selectedColor = (Color)e.Item.Tag;

			this._lastSelectedTextForecolor = selectedColor;
			this.RichTextEditor.Selection.ApplyTextForecolor(selectedColor);
		}
		#endregion //forecolorText_ColorClick

		#region highlightColorTool_Opening
		private void highlightColorTool_Opening(object sender, ToolOpeningEventArgs e)
		{
			// Initialize the checked state of the GalleryItems based on the current
			// CharacterSettings Color
			CharacterSettings	characterSettings	= this.richTextEditorStudioUserControl.CurrentCharacterSettings;
			HighlightColor?		colorSetting		= false == characterSettings.HighlightColor.HasValue ? null : characterSettings.HighlightColor;

			GalleryTool			galleryTool			= this.highlightColorTool.Items[0] as GalleryTool;
			foreach (GalleryItem galleryItem in galleryTool.Items)
			{
				galleryItem.IsSelected = null != colorSetting && colorSetting == RichTextEditorStudioUserControl.HighlightColorFromColor((Color)galleryItem.Tag);
			}

		}
		#endregion //highlightColorTool_Opening

		#region highlightColorTool_Click
		private void highlightColorTool_Click(object sender, RoutedEventArgs e)
		{
			if (false == this.highlightColorTool.IsChecked)
				this.RichTextEditor.Selection.ApplyTextHighlightColor(HighlightColor.None);
			else
				this.RichTextEditor.Selection.ApplyTextHighlightColor(this._lastSelectedTextHighlightColor);
		}
		#endregion //highlightColorTool_Click

		#region highlightColor_ColorClick
		private void highlightColor_ColorClick(object sender, GalleryItemEventArgs e)
		{
			if (e.Item == null || e.Item.Tag == null)
				return;

			Color selectedColor = (Color)e.Item.Tag;

			this._lastSelectedTextHighlightColor = RichTextEditorStudioUserControl.HighlightColorFromColor(selectedColor);

			this.RichTextEditor.Selection.ApplyTextHighlightColor(this._lastSelectedTextHighlightColor);
		}
		#endregion //highlightColor_ColorClick

		#region lineSpacingTool_Opening
		private void lineSpacingTool_Opening(object sender, ToolOpeningEventArgs e)
		{
			// Initialize the checked state of the togglebutton tools in the menu based on the current
			// ParagraphSettings LineFactor
			ParagraphSettings paragraphSettings = this.RichTextEditorStudioUserControl.CurrentParagraphSettings;

			float lineSpacingFactor = -1;
			if (paragraphSettings.Spacing != null &&
				paragraphSettings.Spacing.LineSpacing.HasValue &&
				paragraphSettings.Spacing.LineSpacing.Value.LineFactor.HasValue)
				lineSpacingFactor = paragraphSettings.Spacing.LineSpacing.Value.LineFactor.Value;

			string lineSpacingMultipleAsString = lineSpacingFactor.ToString();
			foreach (ToggleButtonTool tool in this.lineSpacingTool.Items)
				tool.IsChecked = tool.Caption == lineSpacingMultipleAsString;
		}
		#endregion //lineSpacingTool_Opening

		#region lineSpacing_Click
		private void lineSpacing_Click(object sender, RoutedEventArgs e)
		{
			ToggleButtonTool clickedTool = sender as ToggleButtonTool;
			float selectedLineSpacingFactor;
			bool success = float.TryParse(clickedTool.Caption, out selectedLineSpacingFactor);

			ParagraphSettings paragraphSettings = new ParagraphSettings();
			ParagraphSpacingSettings paragraphSpacingSettings = new ParagraphSpacingSettings();

			paragraphSettings.Spacing = paragraphSpacingSettings;
			paragraphSpacingSettings.LineSpacing = new LineSpacing(selectedLineSpacingFactor);

			string error;
			this.RichTextEditor.Document.ApplyParagraphSettings(this.RichTextEditor.Selection.DocumentSpan, paragraphSettings, out error);
		}
		#endregion //lineSpacing_Click

		#region noParagraphShadingTool_Click
		private void noParagraphShadingTool_Click(object sender, RoutedEventArgs e)
		{
			this.RichTextEditor.Selection.ApplyParagraphShading(null);
		}
		#endregion //noParagraphShadingTool_Click

		#region numbering_Click
		private void numbering_Click(object sender, RoutedEventArgs e)
		{
			ToggleButtonTool clickedTool = sender as ToggleButtonTool;

			if (null != clickedTool)
			{
				string styleId = (string)clickedTool.Tag;
				if (string.IsNullOrEmpty(styleId))
					this.RichTextEditor.Selection.ClearParagraphListStyle();
				else
					this.RichTextEditor.Selection.ApplyParagraphListStyle(styleId);

				this._lastSelectedNumberingStyleId = styleId;
			}
		}
		#endregion //numbering_Click

		#region numberingTool_Click
		private void numberingTool_Click(object sender, RoutedEventArgs e)
		{
			if (this.numberingTool.IsChecked)
				this.RichTextEditor.Selection.ApplyParagraphListStyle(this._lastSelectedNumberingStyleId);
			else
				this.RichTextEditor.Selection.ClearParagraphListStyle();
		}
		#endregion //numberingTool_Click

		#region paragraphShadingTool_Opening
		private void paragraphShadingTool_Opening(object sender, ToolOpeningEventArgs e)
		{
			// Initialize the checked state of the GalleryItems based on the current
			// ParagraphSettings Shading Background Color
			ParagraphSettings	paragraphSettings	= this.RichTextEditorStudioUserControl.CurrentParagraphSettings;
			Color?				colorSetting		= false == paragraphSettings.Shading.HasValue || false == paragraphSettings.Shading.Value.Background.Color.HasValue ?
										null : paragraphSettings.Shading.Value.Background.Color;

			GalleryTool			galleryTool = this.paragraphShadingTool.Items[0] as GalleryTool;
			foreach (GalleryItem galleryItem in galleryTool.Items)
			{
				galleryItem.IsSelected = null != colorSetting &&
								 colorSetting == (Color)galleryItem.Tag;
			}
		}
		#endregion //paragraphShadingTool_Opening

		#region paragraphShadingTool_Click
		private void paragraphShadingTool_Click(object sender, RoutedEventArgs e)
		{
			if (this.paragraphShadingTool.IsChecked)
			{
				Shading shading = new Shading(new ColorInfo(this._lastSelectedParagraphShadingColor));
				this.RichTextEditor.Selection.ApplyParagraphShading(shading);
			}
		}
		#endregion //paragraphShadingTool_Click

		#region paragraphShading_ColorClick
		private void paragraphShading_ColorClick(object sender, GalleryItemEventArgs e)
		{
			if (e.Item == null || e.Item.Tag == null)
				return;

			Color selectedColor = (Color)e.Item.Tag;

			this._lastSelectedParagraphShadingColor = selectedColor;

			Shading shading = new Shading(new ColorInfo(this._lastSelectedParagraphShadingColor));
			this.RichTextEditor.Selection.ApplyParagraphShading(shading);
		}
		#endregion //paragraphShading_ColorClick

		#region RichTextEditor_ActiveDocumentViewChanged
		void RichTextEditor_ActiveDocumentViewChanged(object sender, ActiveDocumentViewChangedEventArgs e)
		{
			this.UpdateToolSettings();
		}
		#endregion //RichTextEditor_ActiveDocumentViewChanged

		#region RichTextEditor_DocumentChanged
		void RichTextEditor_DocumentChanged(object sender, DocumentChangedEventArgs e)
		{
			if (e.NewDocument == null)
				return;

			this.InitializeBulletAndNumberingTools();
			this.UpdateToolSettings();
		}
		#endregion //RichTextEditor_DocumentChanged

		#region richTextEditorStudioUserControl_DocumentContentOrSelectionChanged
		void richTextEditorStudioUserControl_DocumentContentOrSelectionChanged(object sender, EventArgs e)
		{
			this.UpdateToolSettings();
		}
		#endregion //richTextEditorStudioUserControl_DocumentContentOrSelectionChanged

		#endregion //Event Handlers
	}
}
